package sortmerge;

import java.lang.*;
import java.util.*;
import java.io.*;

/**
 * Big, fat Sort input class. This is actually what the 
 * heap deals with, but due to lack of foresight in our 
 * design, this class has had all sorts of features bolted
 * on to it to make this program go.
 */
class SortInput implements Comparable<SortInput>
{
  protected String current_string = null;
  protected FileInputStream istream = null;
  protected BufferedReader reader = null;
  protected boolean marked_input = false;

  /**
   * Construct a new SortInput class using the supplied file input
   * and get the first item from it.
   * @param input The file to use as input.
   */
  public SortInput (FileInputStream input) throws java.io.EOFException, java.io.IOException
  {
    istream = input;
    reader = new BufferedReader (new FileReader (istream.getFD()));
    next();
  }

  /**
   * Construct a new SortInput but rather than use a file as input.
   * This will render the next method and such useless.
   * @param instring The value to initialise the current string to.
   * @param marked_state The state to mark the item as.
   */
  public SortInput (String instring, boolean marked_state)
  {
    current_string = instring;
    setMarkedState (marked_state);
  }

  /**
   * Get this sort input as being "marked" or flagged. 
   * @return the marked status of this input item.
   */
  public boolean getMarkedState()
  {
    return marked_input;
  }

  /**
   * Set the marked state for this input item. This is used
   * in replacement-selection stuff to indicate that this item is
   * intended to go into the next run.
   * @param marked_state the value to set as the marked state.
   */
  public void setMarkedState(boolean marked_state)
  {
    marked_input = marked_state;
  }

  /**
   * Get the next string from the input stream and return it.
   * This also stores it in the class and will be returned by calling
   * getString().
   * @return The string read from the class.
   */
  public String next() throws java.io.EOFException, java.io.IOException 
  {
    assert (istream != null);
    String new_string = reader.readLine();

    if (new_string == null)
      {
	throw new EOFException ("End of input file reached");
      }

    current_string = new_string;
    
    return current_string;
  }

  /**
   * Get the string that was last read from the input file.
   * @return The last read string.
   */
  public String getString()
  {
    return current_string;
  }

  /**
   * Set the current string. This is intended to be used when we
   * are not using a file as input. In fact, the program will fail
   * an assertion if this has been initialised to use a file as input.
   * @param newstring The new value for the current string.
   */
  public void setString (String newstring)
  {
    assert (istream == null);
    current_string = newstring;
  }

  /**
   * Compare the current string in this sort input with the supplied
   * string.
   * @param s The SortInput to compare to.
   */
  public int compareTo(SortInput s)
  {
    //If either this item or the one we are comparing to is marked,
    //then the order should be adjusted so that marked items are always
    //greater than unmarked input.
    if (marked_input && !s.getMarkedState()) 
      {
	return 1;
      }
    else if (!marked_input && s.getMarkedState())
      {
	return -1;
      }
    
    return current_string.compareTo (s.getString());
  }
}
